package br.com.mbamobi.sewoo-printer;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;

import com.getcapacitor.JSObject;
import com.getcapacitor.NativePlugin;
import com.getcapacitor.Plugin;
import com.getcapacitor.PluginCall;
import com.getcapacitor.PluginMethod;

import com.sewoo.jpos.printer.CPCLPrinter;
import com.sewoo.port.android.BluetoothPort;
import com.sewoo.request.android.RequestHandler;

import org.json.JSONArray;

import java.io.IOException;
import java.util.Set;

@NativePlugin()
public class SewooPlugin extends Plugin {

    public CPCLPrinter printer = new CPCLPrinter();
    public Thread hThread = new Thread();
    public BluetoothPort port = new BluetoothPort();
    public String address;

    /***
     * Bluetooth devices disponíveis
     * @return
     */
    @PluginMethod()
    public void devices(PluginCall call) {
        JSONArray printers = new JSONArray();
        JSObject btDevices = new JSObject();


        try {
            BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
            Set<BluetoothDevice> devices = adapter.getBondedDevices();

            for (BluetoothDevice device : devices) {
                String name = device.getName();
                String mac = device.getAddress();

                JSObject p = new JSObject();
                p.put("name", name);
                p.put("address", mac);
                printers.put(p);
            }

            btDevices.put("devices", printers);
        } catch (Exception e) {
            System.err.println(e.getMessage());
            call.error(e.getMessage());
        }

        call.success(btDevices);
    }

    /***
     * Impressora connectada?
     * @return
     */
    @PluginMethod()
    public void isConnected(PluginCall call) {
        JSObject result = new JSObject();

        result.put("connected", port.isConnected());
        call.success(result);
    }

    /***
     * Conectar com o dispositivo
     * @return
     */
    @PluginMethod()
    public void connect(PluginCall call) {
        address = call.getString("address");

        try {
            this.port.connect(address);
            RequestHandler rh = new RequestHandler();

            hThread = new java.lang.Thread(rh);
            hThread.start();

            System.out.println("::::::::::::::::::::::::::::::::::");
            System.out.println("BT Conectado: " + address);
            System.out.println("::::::::::::::::::::::::::::::::::");

            call.success();

        } catch (Exception e) {
            System.err.println(e.getMessage());
            call.error(e.getMessage());
        }
    }

    /***
     * Desconectar com o dispositivo
     * @return
     */
    @PluginMethod()
    public void disconnect(PluginCall call) {
        if (port != null) {
            try {
                port.disconnect();

                if ((hThread != null) && (hThread.isAlive())) {
                    hThread.interrupt();
                }

                System.out.println("::::::::::::::::::::::::::::::::::");
                System.out.println("BT Desconectado: " + address);
                System.out.println("::::::::::::::::::::::::::::::::::");

                call.success();
            } catch (Exception e) {
                System.err.println(e.getMessage());
                call.error(e.getMessage());
            }
        }
    }

    @PluginMethod()
    public void print(PluginCall call) {
        String cpcl = call.getString("cpcl");

        if (port.isConnected()) {
            try {
                printer.userString(cpcl);

                System.out.println("::::::::::::::::::::::::::::::::::");
                System.out.print(printer.getBuffer());
                System.out.println("::::::::::::::::::::::::::::::::::");

                printer.write(printer.getBuffer().toString().getBytes("ISO-8859-1"));
                call.success();
            } catch (Exception e) {
                System.err.println(e.getMessage());
                call.error(e.getMessage());
            }
        }
        else {
            System.err.println("Verifique a conexão com a impressora");
            call.error("Verifique a conexão com a impressora");
        }
    }

}
